﻿Imports System.Drawing
Imports System.Text
Imports System.Math

Namespace Argus.Imaging

    Public Class BitmapToText
        Implements IDisposable

        '*********************************************************************************************************************
        '
        '             Class:  Bitmap2Text
        '      Initial Date:  10/03/2008
        '      Last Updated:  10/09/2008
        '     Programmer(s):  Blake Pell
        '       Description:  This class will allow you to turn a picture into either ASCII Art, colored HTML Art or 
        '                     Unicode Art.  
        '
        '*********************************************************************************************************************

        ''' <summary>
        ''' Darkest to lightest characters to use in Unicode encoding.  
        ''' </summary>
        ''' <remarks>
        ''' For this to work, a font with a constant width needs to be used and moreso, this was specifically written for Courier New
        ''' was specifically written for Courier New (1.5 font works well for display).  You can change the characters if you want
        ''' different fonts.  New characters can be added and the CalculateBrightnessValue will compenstate.  For a good render though
        ''' you'll go darkest looking to lightest looking characters
        ''' </remarks>
        ''' 
        Private _unicodeCharArray = New String() {"█", "█", "▓", "▓", "▒", "▒", "░", "░", "#", "$", "@", "%", "&", "9", "W", "~", ",", ".", "`", "`", " "}

        ''' <summary>
        ''' Darkest to lightest characters to use in ASCII encoding.  
        ''' </summary>
        ''' <remarks>
        ''' Same remarks as above.
        ''' </remarks>
        ''' 
        Private _asciiCharArray = New String() {"$", "$", "#", "#", "&", "&", "B", "B", "2", "2", "!", "!", "b", "b", "c", "c", "'", "'", "`", "`", " "}

        Public Sub New(ByVal sourceBitmap As Bitmap)
            Me.SourceBitmap = sourceBitmap
        End Sub

        Public Sub New()

        End Sub

        ''' <summary>
        ''' Calculates the brightness based off of the bitmap and the number of elements in our character array to choose
        ''' from to create the picture.
        ''' </summary>
        ''' <param name="PixelBrightness"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Private Function CalculateBrightnessValue(ByVal PixelBrightness As Single) As Integer
            'CalculateBrightnessValue = Ceiling(PixelBrightness * 100) / UBound(_charArray)
            CalculateBrightnessValue = Floor((PixelBrightness * 100) / 5) '/ UBound(_charArray)
        End Function

        ''' <summary>
        ''' Resizes the bitmap set via the source bitmap property to the specified heighth and width.
        ''' </summary>
        ''' <param name="width"></param>
        ''' <param name="height"></param>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Private Function ResizeBitmap(ByVal width As Integer, ByVal height As Integer) As Bitmap
            Dim destBitmap As New Bitmap(width, height, System.Drawing.Imaging.PixelFormat.Format24bppRgb)
            destBitmap.SetResolution(_sourceBitmap.HorizontalResolution, _sourceBitmap.VerticalResolution)

            Dim gr As Graphics = Graphics.FromImage(destBitmap)
            gr.InterpolationMode = Drawing2D.InterpolationMode.HighQualityBicubic
            gr.DrawImage(_sourceBitmap, New Rectangle(0, 0, width, height), New Rectangle(0, 0, _sourceBitmap.Width, _sourceBitmap.Height), GraphicsUnit.Pixel)
            gr.Dispose()
            Return destBitmap
        End Function

        Private _sourceBitmap As Bitmap
        ''' <summary>
        ''' The source bitmap.  
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property SourceBitmap() As Bitmap
            Get
                Return _sourceBitmap
            End Get
            Set(ByVal value As Bitmap)

                If _sourceBitmap IsNot Nothing Then
                    _sourceBitmap.Dispose()
                End If

                _sourceBitmap = value

                If _autoResize = True Then
                    _sourceBitmap = ResizeBitmap(640 / 4, 480 / 4)
                End If

            End Set
        End Property

        ''' <summary>
        ''' For use in the AsciiText and UnicodeText properties
        ''' </summary>
        ''' <remarks></remarks>
        Private _sb As New StringBuilder

        ''' <summary>
        ''' The text that makes up the picture.  This re-renders the image when called so if you plan on manipulating the text you
        ''' will need to save the string in a local variable.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        Public ReadOnly Property UnicodeText() As String
            Get

                If _sourceBitmap Is Nothing Then
                    Return String.Empty
                End If

                _sb = New StringBuilder

                For y As Integer = 0 To _sourceBitmap.Height - 1
                    For x As Integer = 0 To _sourceBitmap.Width - 1
                        _sb.AppendFormat("{0}", _unicodeCharArray(CalculateBrightnessValue(_sourceBitmap.GetPixel(x, y).GetBrightness)))
                    Next
                    _sb.Append(vbCrLf)
                Next

                Return _sb.ToString
            End Get
        End Property

        ''' <summary>
        ''' The text that makes up the picture.  This re-renders the image when called so if you plan on manipulating the text you
        ''' will need to save the string in a local variable.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks>
        ''' </remarks>
        Public ReadOnly Property AsciiText() As String
            Get

                If _sourceBitmap Is Nothing Then
                    Return String.Empty
                End If

                _sb = New StringBuilder

                For y As Integer = 0 To _sourceBitmap.Height - 1
                    For x As Integer = 0 To _sourceBitmap.Width - 1
                        _sb.AppendFormat("{0}", _asciiCharArray(CalculateBrightnessValue(_sourceBitmap.GetPixel(x, y).GetBrightness)))
                    Next
                    _sb.Append(vbCrLf)
                Next

                Return _sb.ToString
            End Get
        End Property

        Private _htmlArtText As String = "HTMLART"
        ''' <summary>
        ''' The text to repeat when making up the HTML Art page.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property HtmlArtText() As String
            Get
                Return _htmlArtText
            End Get
            Set(ByVal value As String)
                _htmlArtText = value
            End Set
        End Property

        ''' <summary>
        ''' The HTML to display the color coded picture.  The text used it set in the HtmlArtText property and the IncludePageTags 
        ''' boolean property specifies whether or not this should render the html and body tags.  It will render them by default.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public ReadOnly Property HtmlArt() As String
            Get

                If _sourceBitmap Is Nothing Then
                    Return String.Empty
                End If

                Dim textPosition As Integer = 0
                _sb = New StringBuilder

                If _IncludePageTags = True Then
                    _sb.AppendFormat("<html><body bgcolor='{0}' link='#FFFFFF' vlink='#CCCCCC'>", System.Drawing.ColorTranslator.ToHtml(_backgroundColor))
                End If

                _sb.Append("<div style='font-family: Courier New; font-size: 8px;'>")

                For y As Integer = 0 To _sourceBitmap.Height - 1
                    For x As Integer = 0 To _sourceBitmap.Width - 1
                        _sb.AppendFormat("<font color='{0}'>{1}</font>", System.Drawing.ColorTranslator.ToHtml(_sourceBitmap.GetPixel(x, y)), _htmlArtText.Substring(textPosition, 1))
                        textPosition += 1

                        If textPosition = _htmlArtText.Length Then
                            textPosition = 0
                        End If

                    Next
                    _sb.Append("<br />")
                Next

                _sb.Append("<br /><p style='color:white;font-size: 12px;'>Photo to HTML Art software written by Blake Pell - <a href='http://www.blakepell.com'>http://www.blakepell.com</a></p>")
                _sb.Append("</div>")

                If _IncludePageTags = True Then
                    _sb.Append("</body></html>")
                End If

                Return _sb.ToString
            End Get
        End Property

        Private _backgroundColor As System.Drawing.Color = Color.Black
        ''' <summary>
        ''' The background color for the HTML page if the IncludePageTags is set to true.
        ''' </summary>
        ''' <value>Default is Color.Black</value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property BackgroundColor() As System.Drawing.Color
            Get
                Return _backgroundColor
            End Get
            Set(ByVal value As System.Drawing.Color)
                _backgroundColor = value
            End Set
        End Property

        Private _autoResize As Boolean = True
        ''' <summary>
        ''' Whether or not to resize original photo into a more manageable size before rendering the text.  This is true by default.
        ''' If this property is toggled to false, you will be responsible for setting the image size yourself or leaving it how it is
        ''' (which can produce very large outputs).
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property AutoResize() As Boolean
            Get
                Return _autoResize
            End Get
            Set(ByVal value As Boolean)
                _autoResize = value
            End Set
        End Property

        Private _IncludePageTags As Boolean = True
        ''' <summary>
        ''' Whether or not the HtmlArt property should return page tags with it.
        ''' </summary>
        ''' <value></value>
        ''' <returns></returns>
        ''' <remarks></remarks>
        Public Property IncludePageTags() As Boolean
            Get
                Return _IncludePageTags
            End Get
            Set(ByVal value As Boolean)
                _IncludePageTags = value
            End Set
        End Property

        Private disposedValue As Boolean = False        ' To detect redundant calls

        ''' <summary>
        ''' Cleanup objects in memory.
        ''' </summary>
        ''' <param name="disposing"></param>
        ''' <remarks></remarks>
        Protected Overridable Sub Dispose(ByVal disposing As Boolean)
            If Not Me.disposedValue Then
                If disposing Then
                    ' TODO: free other state (managed objects).
                    If _sourceBitmap IsNot Nothing Then
                        _sourceBitmap.Dispose() : _sourceBitmap = Nothing
                    End If
                End If

                ' TODO: free your own state (unmanaged objects).
                ' TODO: set large fields to null.
            End If
            Me.disposedValue = True
        End Sub

#Region " IDisposable Support "
        ' This code added by Visual Basic to correctly implement the disposable pattern.
        Public Sub Dispose() Implements IDisposable.Dispose
            ' Do not change this code.  Put cleanup code in Dispose(ByVal disposing As Boolean) above.
            Dispose(True)
            GC.SuppressFinalize(Me)
        End Sub
#End Region

    End Class

End Namespace
